window.Gleam.initSearch([{"doc":"lenient_parse","title":"lenient_parse","content":"# lenient_parse\n\n[![Package Version](https://img.shields.io/hexpm/v/lenient_parse)](https://hex.pm/packages/lenient_parse)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lenient_parse/)\n\nA Gleam library that replicates the functionality of Python&#39;s built-in `float()`\nand `int()` functions for parsing strings into float and integer values. This\npackage offers more flexible parsing than the standard Gleam functions.\n\n- `float(&quot;3.14&quot;)` -&gt; `&quot;3.14&quot; |&gt; lenient_parse.to_float`\n- `int(&quot;42&quot;)` -&gt; `&quot;42&quot; |&gt; lenient_parse.to_int`\n- `int(&quot;1010&quot;, base=2)` -&gt; `&quot;1010&quot; |&gt; lenient_parse.to_int_with_base(2)`\n\n## Installation\n\n```sh\ngleam add lenient_parse\n```\n\n## Usage\n\n```gleam\nimport gleam/float\nimport gleam/int\nimport gleam/io\nimport lenient_parse\n\npub fn main() {\n  // Parse a string containing an integer value into a float\n\n  &quot;1&quot; |&gt; lenient_parse.to_float |&gt; io.debug // Ok(1.0)\n  &quot;1&quot; |&gt; float.parse |&gt; io.debug // Error(Nil)\n\n  // Parse a string containing a negative float\n\n  &quot;-5.001&quot; |&gt; lenient_parse.to_float |&gt; io.debug // Ok(-5.001)\n  &quot;-5.001&quot; |&gt; float.parse |&gt; io.debug // Ok(-5.001)\n\n  // Parse a string containing a complex float with scientific notation\n\n  &quot;-1_234.567_8e-2&quot; |&gt; lenient_parse.to_float |&gt; io.debug // Ok(-12.345678)\n  &quot;-1_234.567_8e-2&quot; |&gt; float.parse |&gt; io.debug // Error(Nil)\n\n  // Parse a string containing an integer\n\n  &quot;123&quot; |&gt; lenient_parse.to_int |&gt; io.debug // Ok(123)\n  &quot;123&quot; |&gt; int.parse |&gt; io.debug // Ok(123)\n\n  // Parse a string containing a negative integer with surrounding whitespace\n\n  &quot;  -123  &quot; |&gt; lenient_parse.to_int |&gt; io.debug // Ok(-123)\n  &quot;  -123  &quot; |&gt; int.parse |&gt; io.debug // Error(Nil)\n\n  // Parse a string containing an integer with underscores\n\n  &quot;1_000_000&quot; |&gt; lenient_parse.to_int |&gt; io.debug // Ok(1000000)\n  &quot;1_000_000&quot; |&gt; int.parse |&gt; io.debug // Error(Nil)\n\n  // Parse a string containing a binary number with underscores\n\n  &quot;1000_0000&quot; |&gt; lenient_parse.to_int_with_base(2) |&gt; io.debug // Ok(128)\n  &quot;1000_0000&quot; |&gt; int.base_parse(2) |&gt; io.debug // Error(Nil)\n\n  // Parse a string containing a hexadecimal number with underscores\n\n  &quot;DEAD_BEEF&quot; |&gt; lenient_parse.to_int_with_base(16) |&gt; io.debug // Ok(3735928559)\n  &quot;DEAD_BEEF&quot; |&gt; int.base_parse(16) |&gt; io.debug // Error(Nil)\n\n  // Use base 0 to automatically detect the base when parsing strings with prefix indicators\n\n  let dates = [\n    &quot;0b11011110000&quot;, &quot;0o3625&quot;, &quot;1865&quot;, &quot;0x7bc&quot;, &quot;0B11110110001&quot;, &quot;1929&quot;,\n    &quot;0O3507&quot;, &quot;0X7a9&quot;, &quot;0b11011111011&quot;,\n  ]\n\n  dates\n  |&gt; list.map(lenient_parse.to_int_with_base(_, 0))\n  |&gt; io.debug()\n  // [Ok(1776), Ok(1941), Ok(1865), Ok(1980), Ok(1969), Ok(1929), Ok(1863), Ok(1961), Ok(1787)]\n\n  dates\n  |&gt; list.map(int.base_parse(_, 0))\n  |&gt; io.debug()\n  // [Error(Nil), Error(Nil), Error(Nil), Error(Nil), Error(Nil), Error(Nil), Error(Nil), Error(Nil), Error(Nil)]\n\n  // Nice errors\n\n  &quot;12.3e_3&quot; |&gt; lenient_parse.to_float |&gt; io.debug // Error(InvalidUnderscorePosition(5))\n  &quot;12.3e_3&quot; |&gt; float.parse |&gt; io.debug // Error(Nil)\n}\n```\n\n## [Rigorous Testing](https://github.com/JosephTLyons/lenient_parse/tree/main/test/data)\n\n`lenient_parse`&#39;s testing is extensive. We test the tokenization step, the\noverall parse procedure, as well as various intermediate layers. We currently\nhave ~600 passing tests. Regressions are **not** welcome here.\n\n### Backed by Python\n\nEach test input is also processed using Python&#39;s (3.13) `float()` and `int()`\nfunctions. We verify that `lenient_parse` produces the same output as Python. If\nPython&#39;s built-ins succeed, `lenient_parse` should also succeed with identical\nresults. If Python&#39;s built-ins fail to parse, `lenient_parse` should also fail.\nThis ensures that `lenient_parse` behaves consistently with Python&#39;s built-ins\nfor all supplied test data. Tests are run against both the Erlang and JavaScript\ntargets.\n\nIf you run into a case where `lenient_parse` and Python&#39;s built-ins disagree,\nplease open an [issue](https://github.com/JosephTLyons/lenient_parse/issues) -\nwe aim to be 100% consistent with Python&#39;s built-ins and we will fix any\nreported discrepancies.\n\n## Development\n\nTo run the tests for this package, you&#39;ll need to [install\n`uv`](https://docs.astral.sh/uv/getting-started/installation/).\n","url":"index.html"},{"doc":"parse_error","title":"ParseError","content":"pub type ParseError {\n  EmptyString\n  WhitespaceOnlyString\n  InvalidUnderscorePosition(index: Int)\n  InvalidDecimalPosition(index: Int)\n  InvalidSignPosition(index: Int, character: String)\n  InvalidDigitPosition(index: Int, character: String)\n  BasePrefixOnly(index_range: #(Int, Int), prefix: String)\n  OutOfBaseRange(\n    index: Int,\n    character: String,\n    value: Int,\n    base: Int,\n  )\n  InvalidExponentSymbolPosition(index: Int, character: String)\n  UnknownCharacter(index: Int, character: String)\n  InvalidBaseValue(base: Int)\n}\n\nEmptyString\n Represents an error when the input string is empty.\n\nWhitespaceOnlyString\n Represents an error when the input string contains only whitespace\n characters.\n\nInvalidUnderscorePosition(index: Int)\n Represents an error when an underscore is in an invalid position within\n the number string.\n\n - `index`: The position of the invalid underscore in the input string.\n\nInvalidDecimalPosition(index: Int)\n Represents an error when a decimal point is in an invalid position within\n the number string.\n\n - `index`: The position of the invalid decimal point in the input string.\n\nInvalidSignPosition(index: Int, character: String)\n Represents an error when a sign (+ or -) is in an invalid position within\n the number string.\n\n - `index`: The position of the invalid sign in the input string.\n - `character`: The sign character that caused the error as a `String`.\n\nInvalidDigitPosition(index: Int, character: String)\n Represents an error when a digit is in an invalid position within the\n number string.\n\n - `index`: The position of the invalid digit in the input string.\n - `character`: The digit character that caused the error as a `String`.\n\nBasePrefixOnly(index_range: #(Int, Int), prefix: String)\n Represents an error when the user specifies base = 0, but the input string\n only contains a base prefix (`0b`, `0o`, 0x), and no value.\n\n - `index_range`: The position range of the prefix in the input string.\n - `prefix`: The base prefix as a `String`.\n\nOutOfBaseRange(\n  index: Int,\n  character: String,\n  value: Int,\n  base: Int,\n)\n Represents an error when a digit character is out of the valid range for\n the specified base.\n\n - `index`: The position of the out-of-range digit in the input string.\n - `character`: The string representation of the out-of-range digit.\n - `value`: The integer value of the out-of-range digit.\n - `base`: The base in which the digit is out of range.\n\nInvalidExponentSymbolPosition(index: Int, character: String)\n Represents an error when an exponent symbol (e or E) is in an invalid\n position within the number string.\n\n - `index`: The position of the invalid exponent symbol in the input\n string.\n - `character`: The exponent symbol that caused the error as a `String`.\n\nUnknownCharacter(index: Int, character: String)\n Represents an error when an invalid character is encountered during\n parsing.\n\n - `index`: The position of the invalid character in the input string.\n - `character`: The invalid character as a `String`.\n\nInvalidBaseValue(base: Int)\n Represents an error when the base provided for parsing is invalid.\n\n - `base`: The invalid base as an `Int`. The base must be between 2 and 36\n inclusive.\n\nSynonyms:\nparse_error.ParseError\nparse_error ParseError","url":"parse_error.html#ParseError"},{"doc":"parse_error","title":"parse_error","content":"","url":"parse_error.html"},{"doc":"lenient_parse","title":"to_float","content":"pub fn to_float(text text: String) -&gt; Result(Float, ParseError)\n Converts a string to a float using a more lenient parsing method than\n gleam&#39;s `float.parse()`. It behaves similarly to Python&#39;s `float()` built-in\n function.\nSynonyms:\nlenient_parse.to_float\nlenient_parse to_float","url":"lenient_parse.html#to_float"},{"doc":"lenient_parse","title":"to_int","content":"pub fn to_int(text text: String) -&gt; Result(Int, ParseError)\n Converts a string to an integer using a more lenient parsing method than\n gleam&#39;s `int.parse()`. It behaves similarly to Python&#39;s `int()` built-in\n function, using a default base of 10.\nSynonyms:\nlenient_parse.to_int\nlenient_parse to_int","url":"lenient_parse.html#to_int"},{"doc":"lenient_parse","title":"to_int_with_base","content":"pub fn to_int_with_base(\n  text text: String,\n  base base: Int,\n) -&gt; Result(Int, ParseError)\n Converts a string to an integer using a more lenient parsing method than\n gleam&#39;s `int.parse()`, allowing for a specified base. It behaves similarly\n to Python&#39;s `int()` built-in function with a base parameter.\nSynonyms:\nlenient_parse.to_int_with_base\nlenient_parse to_int_with_base","url":"lenient_parse.html#to_int_with_base"},{"doc":"lenient_parse","title":"lenient_parse","content":"","url":"lenient_parse.html"}]);